@@ -51,7 +51,19 @@ def set_gelu(version):
         keras.utils.get_custom_objects()['gelu'] = gelu_tanh
 
 
-def piecewise_linear(t, schedule):
+def infinity():
+    """返回默认的代表无穷大的数值
+    """
+    return keras.utils.get_custom_objects().get('infinity', 1e12)
+
+
+def set_infinity(value):
+    """设置新的代表无穷大的数值
+    """
+    keras.utils.get_custom_objects()['infinity'] = value
+
+
+def piecewise_linear(t, schedule, from_zero=True):
     """分段线性函数
     其中schedule是形如{1000: 1, 2000: 0.1}的字典，
     表示 t ∈ [0, 1000]时，输出从0均匀增加至1，而
@@ -59,11 +71,11 @@ def piecewise_linear(t, schedule):
     t > 2000时，保持0.1不变。
     """
     schedule = sorted(schedule.items())
-    if schedule[0][0] != 0:
+    if from_zero and schedule[0][0] != 0:
         schedule = [(0, 0.0)] + schedule
 
-    x = K.constant(schedule[0][1], dtype=K.floatx())
     t = K.cast(t, K.floatx())
+    x = (t * 0 + 1) * schedule[0][1]
     for i in range(len(schedule)):
         t_begin = schedule[i][0]
         x_begin = x
@@ -73,7 +85,7 @@ def piecewise_linear(t, schedule):
             slope = 1.0 * dx / dt
             x = schedule[i][1] + slope * (t - t_begin)
         else:
-            x = K.constant(schedule[i][1], dtype=K.floatx())
+            x = (t * 0 + 1) * schedule[i][1]
         x = K.switch(t >= t_begin, x, x_begin)
 
     return x
@@ -126,9 +138,9 @@ def sequence_masking(x, mask, value=0.0, axis=None):
         if K.dtype(mask) != K.dtype(x):
             mask = K.cast(mask, K.dtype(x))
         if value == '-inf':
-            value = -1e12
+            value = -K.infinity()
         elif value == 'inf':
-            value = 1e12
+            value = K.infinity()
         if axis is None:
             axis = 1
         elif axis < 0:
@@ -186,6 +198,12 @@ def divisible_temporal_padding(x, n):
     return K.temporal_padding(x, (0, p_len))
 
 
+def root_mean_square(x, axis=None, keepdims=False):
+    """均方根，相当于模长的变体
+    """
+    return K.sqrt(K.mean(K.square(x), axis=axis, keepdims=keepdims))
+
+
 def swish(x):
     """swish函数（这样封装过后才有 __name__ 属性）
     """
@@ -227,8 +245,8 @@ def multilabel_categorical_crossentropy(y_true, y_pred):
         4. 详情请看：https://kexue.fm/archives/7359 。
     """
     y_pred = (1 - 2 * y_true) * y_pred
-    y_pred_neg = y_pred - y_true * 1e12
-    y_pred_pos = y_pred - (1 - y_true) * 1e12
+    y_pred_neg = y_pred - y_true * K.infinity()
+    y_pred_pos = y_pred - (1 - y_true) * K.infinity()
     zeros = K.zeros_like(y_pred[..., :1])
     y_pred_neg = K.concatenate([y_pred_neg, zeros], axis=-1)
     y_pred_pos = K.concatenate([y_pred_pos, zeros], axis=-1)
@@ -320,14 +338,19 @@ def recompute_grad(call):
     return inner
 
 
-# 给旧版本keras新增symbolic方法（装饰器），
-# 以便兼容optimizers.py中的代码
+# 给旧版keras新增symbolic（装饰器），以兼容optimizers.py
 K.symbolic = getattr(K, 'symbolic', None) or symbolic
 
+# 添加到 keras.backend 上，使其可以像 K.epsilon() 那样操作
+K.infinity = infinity
+K.set_infinity = set_infinity
+sys.modules['keras.backend'] = K
+
 custom_objects = {
     'gelu_erf': gelu_erf,
     'gelu_tanh': gelu_tanh,
     'gelu': gelu_erf,
+    'root_mean_square': root_mean_square,
     'swish': swish,
     'leaky_relu': leaky_relu,
     'Sinusoidal': Sinusoidal,
