@@ -2,6 +2,7 @@
 # 主要模型
 
 import numpy as np
+from bert4keras.backend import sequence_masking
 from bert4keras.layers import *
 from bert4keras.snippets import insert_arguments
 from bert4keras.snippets import delete_arguments
@@ -23,6 +24,7 @@ class Transformer(object):
         intermediate_size,  # FeedForward的隐层维度
         hidden_act,  # FeedForward隐层的激活函数
         dropout_rate=None,  # Dropout比例
+        attention_dropout_rate=None,  # Attention矩阵的Dropout比例
         embedding_size=None,  # 是否指定embedding_size
         attention_head_size=None,  # Attention中V的head_size
         attention_key_size=None,  # Attention中Q,K的head_size
@@ -49,6 +51,7 @@ class Transformer(object):
         self.attention_key_size = attention_key_size or self.attention_head_size
         self.intermediate_size = intermediate_size
         self.dropout_rate = dropout_rate or 0
+        self.attention_dropout_rate = attention_dropout_rate or 0
         self.hidden_act = hidden_act
         self.embedding_size = embedding_size or hidden_size
         self.sequence_length = sequence_length
@@ -366,7 +369,7 @@ class LM_Mask(object):
                 idxs = K.arange(0, seq_len)
                 mask = idxs[None, :] <= idxs[:, None]
                 mask = K.cast(mask, K.floatx())
-                return -(1 - mask[None, None]) * 1e12
+                return -(1 - mask[None, None]) * K.infinity()
 
             self.attention_bias = self.apply(
                 inputs=self.inputs[0],
@@ -392,7 +395,7 @@ class UniLM_Mask(object):
                 idxs = K.cumsum(s, axis=1)
                 mask = idxs[:, None, :] <= idxs[:, :, None]
                 mask = K.cast(mask, K.floatx())
-                return -(1 - mask[:, None]) * 1e12
+                return -(1 - mask[:, None]) * K.infinity()
 
             self.attention_bias = self.apply(
                 inputs=self.inputs[1],
@@ -558,6 +561,7 @@ class BERT(Transformer):
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
             key_size=self.attention_key_size,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -808,6 +812,7 @@ class ALBERT(BERT):
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
             key_size=self.attention_key_size,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -1028,6 +1033,7 @@ class NEZHA(BERT):
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
             key_size=self.attention_key_size,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -1131,6 +1137,7 @@ class RoFormer(NEZHA):
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
             key_size=self.attention_key_size,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -1468,6 +1475,7 @@ class GPT2(GPT):
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
             key_size=self.attention_key_size,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -1636,6 +1644,7 @@ class GPT2_ML(GPT):
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
             key_size=self.attention_key_size,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -1945,6 +1954,7 @@ class T5_Encoder(T5_Base):
             key_size=self.attention_key_size,
             use_bias=False,
             attention_scale=False,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=attention_name
         )
@@ -2015,6 +2025,12 @@ class T5_Encoder(T5_Base):
             rate=self.dropout_rate,
             name='Encoder-Output-Dropout'
         )
+        x = self.apply(
+            inputs=x,
+            layer=Lambda,
+            function=sequence_masking,
+            name='Encoder-Output-Masked'
+        )
 
         return x
 
@@ -2134,6 +2150,7 @@ class T5_Decoder(LM_Mask, T5_Base):
             key_size=self.attention_key_size,
             use_bias=False,
             attention_scale=False,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=self_attention_name
         )
@@ -2173,6 +2190,7 @@ class T5_Decoder(LM_Mask, T5_Base):
             key_size=self.attention_key_size,
             use_bias=False,
             attention_scale=False,
+            attention_dropout=self.attention_dropout_rate,
             kernel_initializer=self.initializer,
             name=cross_attention_name
         )
@@ -2245,9 +2263,8 @@ class T5_Decoder(LM_Mask, T5_Base):
         )
         x = self.apply(
             inputs=x,
-            layer=Lambda,
-            function=lambda x: x / self.hidden_size**0.5,
-            mask=lambda i, m: m,
+            layer=Scale,
+            scale=self.hidden_size**(-0.5),
             name='Decoder-Output-Scale'
         )
 
@@ -2399,6 +2416,10 @@ def build_transformer_model(
         configs['max_position'] = configs.get('max_position_embeddings', 512)
     if 'dropout_rate' not in configs:
         configs['dropout_rate'] = configs.get('hidden_dropout_prob')
+    if 'attention_dropout_rate' not in configs:
+        configs['attention_dropout_rate'] = configs.get(
+            'attention_probs_dropout_prob'
+        )
     if 'segment_vocab_size' not in configs:
         configs['segment_vocab_size'] = configs.get('type_vocab_size', 2)
 
