@@ -2,6 +2,7 @@
 # 代码合集
 
 import os, sys, six, re, json
+import unicodedata
 import logging
 import numpy as np
 from collections import defaultdict
@@ -79,6 +80,17 @@ def convert_to_str(text, encoding='utf-8', errors='ignore'):
     return text
 
 
+def lowercase_and_normalize(text):
+    """转小写，并进行简单的标准化
+    """
+    if is_py2:
+        text = unicode(text)
+    text = text.lower()
+    text = unicodedata.normalize('NFD', text)
+    text = ''.join([ch for ch in text if unicodedata.category(ch) != 'Mn'])
+    return text
+
+
 class open:
     """模仿python自带的open函数
     作用：1.主要是为了同时兼容py2和py3；2.增加了索引功能，方便读取大文件。
@@ -186,20 +198,15 @@ class open:
         self.close()
 
 
-def parallel_apply(
-    func,
-    iterable,
-    workers,
-    max_queue_size,
-    callback=None,
-    dummy=False,
-    random_seeds=True
+def parallel_apply_generator(
+    func, iterable, workers, max_queue_size, dummy=False, random_seeds=True
 ):
     """多进程或多线程地将func应用到iterable的每个元素中。
     注意这个apply是异步且无序的，也就是说依次输入a,b,c，但是
-    输出可能是func(c), func(a), func(b)。
+    输出可能是func(c), func(a), func(b)。结果将作为一个
+    generator返回，其中每个item是输入的序号以及该输入对应的
+    处理结果。
     参数：
-        callback: 处理单个输出的回调函数；
         dummy: False是多进程/线性，True则是多线程/线性；
         random_seeds: 每个进程的随机种子。
     """
@@ -229,21 +236,6 @@ def parallel_apply(
     # 启动多进程/线程
     pool = Pool(workers, worker_step, (in_queue, out_queue))
 
-    if callback is None:
-        results = []
-
-    # 后处理函数
-    def process_out_queue():
-        out_count = 0
-        for _ in range(out_queue.qsize()):
-            i, d = out_queue.get()
-            out_count += 1
-            if callback is None:
-                results.append((i, d))
-            else:
-                callback(d)
-        return out_count
-
     # 存入数据，取出结果
     in_count, out_count = 0, 0
     for i, d in enumerate(iterable):
@@ -253,18 +245,54 @@ def parallel_apply(
                 in_queue.put((i, d), block=False)
                 break
             except six.moves.queue.Full:
-                out_count += process_out_queue()
+                for _ in range(out_queue.qsize()):
+                    yield out_queue.get()
+                    out_count += 1
         if in_count % max_queue_size == 0:
-            out_count += process_out_queue()
+            for _ in range(out_queue.qsize()):
+                yield out_queue.get()
+                out_count += 1
 
     while out_count != in_count:
-        out_count += process_out_queue()
+        for _ in range(out_queue.qsize()):
+            yield out_queue.get()
+            out_count += 1
 
     pool.terminate()
 
+
+def parallel_apply(
+    func,
+    iterable,
+    workers,
+    max_queue_size,
+    callback=None,
+    dummy=False,
+    random_seeds=True,
+    unordered=True
+):
+    """多进程或多线程地将func应用到iterable的每个元素中。
+    注意这个apply是异步且无序的，也就是说依次输入a,b,c，但是
+    输出可能是func(c), func(a), func(b)。
+    参数：
+        callback: 处理单个输出的回调函数；
+        dummy: False是多进程/线性，True则是多线程/线性；
+        random_seeds: 每个进程的随机种子；
+        unordered: 若为False，则按照输入顺序返回，仅当callback为None时生效。
+    """
+    generator = parallel_apply_generator(
+        func, iterable, workers, max_queue_size, dummy, random_seeds
+    )
+
     if callback is None:
-        results = sorted(results, key=lambda r: r[0])
-        return [r[1] for r in results]
+        if unordered:
+            return [d for i, d in generator]
+        else:
+            results = sorted(generator, key=lambda d: d[0])
+            return [d for i, d in results]
+    else:
+        for d in generator:
+            callback(d)
 
 
 def sequence_padding(inputs, length=None, value=0, seq_dims=1, mode='post'):
@@ -404,6 +432,11 @@ class DataGenerator(object):
             for d in self.__iter__(random):
                 yield d
 
+    def fortest(self, random=False):
+        while True:
+            for d in self.__iter__(random):
+                yield d[0]
+
     def to_dataset(self, types, shapes, names=None, padded_batch=False):
         """转为tf.data.Dataset格式
         如果传入names的话，自动把数据包装成dict形式。
